/*
Push-And-Pull Switching: Windows Switching Technique
Copyright (C) 2010 Quan Xu
E-mail:xuquan_sizuo@hotmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.Drawing;

using Common;

namespace DesktopWindows
{
    public class Desktop:Dictionary<int,Window>
    {
        private static IntPtr DefinitionActiveHandle = IntPtr.Zero;

        private void GetAllTopWindows()
        {
            //display all top windows

            uint zorder = 0;

            IntPtr Hwnd = User32.GetTopWindow(IntPtr.Zero);
            if (Hwnd == IntPtr.Zero)
            {
                Debug.Assert(false);
                return;
            }
            while (Hwnd != IntPtr.Zero)
            {
                if (User32.IsWindowVisible(Hwnd))
                {
                    Int64 property = User32.GetWindowLong(Hwnd, WindowStyles.GWL_STYLE);
                    if (((property & WindowStyles.WS_VISIBLE) != 0) && ((property & WindowStyles.WS_OVERLAPPEDWINDOW) != 0))
                    {
                        zorder = (uint)this.Count;

                        if (this.ContainsKey(Hwnd.ToInt32()))
                        {
                            return;
                        }
                        else
                        {
                            if (Hwnd == DefinitionActiveHandle)
                            {
                                this.Add(Hwnd.ToInt32(), new Window(Hwnd, zorder, true));
                            }
                            else
                            {
                                this.Add(Hwnd.ToInt32(), new Window(Hwnd, zorder, false));
                            }
                        }
                    }
                }

                Hwnd = User32.GetWindow(Hwnd, GW.HWNDNEXT);
            }
        }

        public static Desktop Empty
        {
            get { return new Desktop(); }
        }

        public static Desktop DesktopWindows
        {
            get 
            { 
                Desktop windows = new Desktop();
                windows.Refresh();
                return windows;
            }
        }

        public void Refresh()
        {
            Clear();
            GetAllTopWindows();
        }

        public Desktop(): base()
        {
        }

        public Window ForegroundWindow
        {
            get
            {
                IntPtr handle = User32.GetForegroundWindow();
                if (this.ContainsKey(handle.ToInt32()))
                {
                    return this[handle.ToInt32()];
                }
                else
                {
                    return null;
                }
            }

            set
            {
                User32.SwitchToThisWindow(value.Handle, false);
            }
        }

        public uint NumberWindows
        {
            get
            {
                return (uint)Desktop.DesktopWindows.Values.Count;
            }
        }

        public Window ActiveWindow
        {
            get
            {
                IntPtr handle = User32.GetActiveWindow();
                if (this.ContainsKey(handle.ToInt32()))
                {
                    return this[handle.ToInt32()];
                }
                else
                {
                    return null;
                }
            }
        }

        //bring hWnd to the top
        public void ChangeWindowOrder(IntPtr hWnd)
        {
            bool isTop = User32.BringWindowToTop(hWnd); //SetForegroundWindow,SetWindowPos
            if (!isTop)
            {
                Console.WriteLine("Bring the window to top is failed.");
            }
        }

        public void SetActiveWindows(IntPtr handle)
        {
            if(handle != IntPtr.Zero)
                DefinitionActiveHandle = handle;
        }

        public Window GetObjWindow(IntPtr handle)
        {
            foreach (Window window in DesktopWindows.Values)
            {
                if (window.Handle == handle)
                    return window;
            }
            return null;
        }

        public Window GetObjWindow(uint m_zorder)
        {
            foreach (Window window in DesktopWindows.Values)
            {
                if (window.ZOrder == m_zorder)
                    return window;
            }
            return null;
        }

        public IntPtr GetDefinitionActiveWindowHandle()
        {
            foreach (Window window in DesktopWindows.Values)
            {
                if (window.IsActive)
                    return window.Handle;
            }
            return IntPtr.Zero;
        }

        public Window GetDefinitionActiveWindow()
        {
            foreach (Window window in DesktopWindows.Values)
            {
                if (window.IsActive)
                    return window;
            }
            return null;
        }

        //intersection
        public Rectangle ChangeWindowtoRectangle(Window window)
        {
            return new Rectangle(window.Left, window.Top, window.Width, window.Height);
        }

        public bool IsIntersection(Window window1, Window window2)
        {
            Rectangle rect1 = ChangeWindowtoRectangle(window1);
            Rectangle rect2 = ChangeWindowtoRectangle(window2);

            return rect1.IntersectsWith(rect2);
        }

        //when the intersection region is small, we consider these two windows are not intersection
        public bool IsIntersection(Window window1, Window window2, double ratio)
        {
            Rectangle rect1 = ChangeWindowtoRectangle(window1);
            Rectangle rect2 = ChangeWindowtoRectangle(window2);

            Rectangle region =  Rectangle.Intersect(rect1, rect2);
            if(region != null)
            {
                int area1 = rect1.Height*rect1.Width;
                int area2 = rect2.Height*rect2.Width;
                if(area1 != 0 && area2 != 0)
                {
                    float r1 = (float)(region.Height * region.Width) / area1;
                    float r2 = (float)(region.Height * region.Width) / area2;

                    if(r1 < ratio && r2 < ratio)
                    {
                        return false;
                    }
                    else
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }
}
