/*
Push-And-Pull Switching: Windows Switching Technique
Copyright (C) 2010 Quan Xu
E-mail:xuquan_sizuo@hotmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using System.Reflection;
using System.Globalization;
using System.Diagnostics;
using System.Timers;
using System.Text;
using System.Collections.Generic;

using Common;
using PushAndPullPGlobalHooks;


namespace DesktopWindows
{
    public enum Status
    {
        Maximize   = 01,
        Minimize   = 02,
        Normal     = 03
    }

    //Window Class
    public class Window
    {
        private IntPtr handle;
        private RECT window_rect;
        private bool sticky;
        private uint z_order;
        private uint process_id;
        private uint thread_id;
        //private string window_name;
        private string class_name;
        //private string module_filename;
        private System.Drawing.Icon icon;

        private bool isActive; //for Push-And-Pull Switching

        static public Window ForegroundWindow()
        {
            IntPtr hWnd = User32.GetForegroundWindow();
            return new Window(hWnd, 0, false);
        }

        public Window(IntPtr handle)
        {
            this.handle = handle;
            this.thread_id = User32.GetWindowThreadProcessId(this.handle, out this.process_id);
        }

        public Window(IntPtr handle, uint z_order, bool _isActive)
        {
            this.handle = handle;
            this.z_order = z_order;
            this.thread_id = User32.GetWindowThreadProcessId(this.handle, out this.process_id);
            User32.GetWindowRect(handle, out this.window_rect);
            //this.window_name = null;
            this.sticky = false;
            //this.module_filename = null;
            this.icon = null;
            this.isActive = _isActive;

            StringBuilder data = new StringBuilder(1024);
            User32.GetClassName(handle, data, data.Capacity);
            this.class_name = data.ToString();
        }

        /// <summary>
        /// Get the window handle.
        /// </summary>
        public IntPtr Handle 
        { 
            get 
            { 
                return handle; 
            } 
        }

        public Status WindowStatus
        {
            get
            {
                if (User32.IsZoomed(Handle))
                    return Status.Maximize;
                else if (User32.IsIconic(Handle))
                    return Status.Minimize;
                else
                    return Status.Normal;
            }
        }

        public Common.RECT Rect
        {
            get
            {
                User32.GetWindowRect(Handle, out window_rect);
                return window_rect;
            }

            set
            {
                if (0 == User32.SetWindowPos(Handle, (IntPtr)Constants.HWND_TOP, value.left, value.top, value.right, value.bottom, 0))
                {
                    throw new Exception("Failed to position window.");
                }
            }
        }

        public int Left
        {
            get
            {
                return Rect.left;
            }
        }

        public int Top
        {
            get
            {
                return Rect.top;
            }
        }

        public int Right
        {
            get
            {
                return Rect.right;
            }
        }

        public int Bottom
        {
            get
            {
                return Rect.bottom;
            }
        }

        public int Width
        {
            get
            {
                return Rect.right - Rect.left;
            }
        }

        public int Height
        {
            get
            {
                return Rect.bottom - Rect.top;
            }
        }

        public int Area
        {
            get
            {
                return Width * Height;
            }
        }

        public uint ZOrder
        {
            get
            {
                return z_order;
            }
            set
            {
                z_order = value;
            }
        }

        public bool IsActive
        {
            get
            {
                return isActive;
            }
            set
            {
                isActive = value;
            }
        }

        /// <summary>
        /// Get the class name.
        /// </summary>
        public string Class
        {
            get
            {
                return this.class_name;
            }
        }

        public uint ProcessId
        {
            get
            {
                return this.process_id;
            }
        }

        public uint ThreadId
        {
            get
            {
                return this.thread_id;
            }
        }

        #region Windows API Helper Functions

        //gets the name of a window from its handle
        public string WindowName
        {
            get
            {
                StringBuilder Title = new StringBuilder(256);
                User32.GetWindowText(Handle, Title, 256);
                return Title.ToString().Trim();
            }
       }

       //gets the name of a window class from a window handle
        public string WindowClass
        {
            get
            {
                StringBuilder Title = new StringBuilder(256);
                User32.RealGetWindowClass(Handle, Title, 256);
                return Title.ToString().Trim();
            }
        }

        //gets the name of a window class from a window handle
        public string WindowClassName
        {
            get
            {
                StringBuilder Title = new StringBuilder(256);
                User32.GetClassName(Handle, Title, 256);
                return Title.ToString().Trim();
            }
        }

        public string GetApplicationName(IntPtr Hwnd,Boolean isFileName)
        {
            // This function gets the name of a application name/filename from a window handle

            StringBuilder m_ApplicationName = new StringBuilder(256);

            UInt32 processId = 0;
            UInt32 threadId = User32.GetWindowThreadProcessId(Handle, out processId);

            //IntPtr hProcess = Kernel32.OpenProcess(ProcessRights.ALL_ACCESS, false, processId);
            IntPtr hProcess = Kernel32.OpenProcess(Constants.PROCESS_QUERY_INFORMATION | Constants.PROCESS_VM_READ, false, processId);
            if (hProcess == IntPtr.Zero)
            {
                return string.Empty;
            }

            IntPtr hMod = IntPtr.Zero;
            int cbNeeded = 0;

            if (User32.EnumProcessModules(hProcess, out hMod, Marshal.SizeOf(hMod), out cbNeeded))
            {
                if(isFileName)
                {
                    User32.GetModuleFileNameEx(hProcess, hMod, m_ApplicationName, 256);
                }
                else
                {
                    User32.GetModuleBaseName(hProcess, hMod, m_ApplicationName, 256);
                }
            }

            Kernel32.CloseHandle(hProcess);

            return m_ApplicationName.ToString().Trim();
        }

        public System.Drawing.Icon Icon
        {
            get
            {
                if (this.icon == null)
                {
                    IntPtr icon_handle = (IntPtr)User32.GetClassLong(this.Handle, Constants.GCL_HICONSM);
                    if (icon_handle != IntPtr.Zero)
                    {
                        this.icon = System.Drawing.Icon.FromHandle(icon_handle);
                    }
                }
                return this.icon == null ? System.Drawing.SystemIcons.Question : this.icon;
            }
        }

        /// <summary>
        /// Get and set the visibility of the window.
        /// </summary>
        public bool Visible
        {
            get
            {
                return User32.IsWindowVisible(Handle);
            }
            set
            {
                if (value)
                {
                    User32.ShowWindow(Handle, Constants.SW_SHOW);
                }
                else if (!sticky)
                {
                    User32.ShowWindow(Handle, Constants.SW_HIDE);
                }
            }
        }

        public IntPtr GetOwner
        {
            get { return User32.GetWindow(Handle, GW.OWNER); }
        }

        public bool IsTopWindow
        {
            get
            {
                Int64 l = User32.GetWindowLong(Handle, WindowStyles.GWL_STYLE);
                return (((l & WindowStyles.WS_VISIBLE) != 0) && ((l & WindowStyles.WS_OVERLAPPEDWINDOW) != 0));
            }
        }

        public bool IsTopMost
        {
            get
            {
                Int64 l = User32.GetWindowLong(Handle, ExtendedStyles.GWL_GLSTYLE);
                return ((l & ExtendedStyles.WS_EX_TOPMOST) != 0);
            }
            set
            {
                User32.SetWindowPos(Handle, (IntPtr)Constants.HWND_TOPMOST, Left, Top, Width, Height, Constants.SWP_NOACTIVATE); //& (Constants.SWP_NOMOVE & Constants.SWP_NOSIZE)
            }
        }

        public bool Foreground
        {
            get
            {
                return (User32.GetForegroundWindow() == Handle);
            }

            set
            {
                if (value)
                {
                    if (!Foreground)
                    {
                        User32.SwitchToThisWindow(Handle, true);
                    }
                }
                else
                {
                    User32.SwitchToThisWindow(User32.GetDesktopWindow(), true);
                }
            }
        }

        #endregion
    } //class Windows
}
